@@ -31,7 +31,7 @@ from requests.exceptions import ConnectionError
 from subprocess import Popen, PIPE
 from ferry.table.prettytable import *
 from ferry.options import CmdHelp
-from ferry.install import Installer, FERRY_HOME
+from ferry.install import Installer, FERRY_HOME, GUEST_DOCKER_REPO
 
 class CLI(object):
     def __init__(self):
@@ -44,14 +44,14 @@ class CLI(object):
         self.cmds.add_option("-k", "--key", "Specify key directory")
         self.cmds.add_option("-m", "--mode", "Deployment mode")
         self.cmds.add_option("-u", "--upgrade", "Upgrade Ferry")
-        self.cmds.add_option("-b", "--build", "Manually build Ferry images")
+        self.cmds.add_option("-b", "--build", "Build Ferry images")
         self.cmds.add_cmd("clean", "Clean zombie Ferry processes")
         self.cmds.add_cmd("server", "Start all the servers")
         self.cmds.add_cmd("deploy", "Deploy a service to the cloud")
         self.cmds.add_cmd("help", "Print this help message")
         self.cmds.add_cmd("info", "Print version information")
         self.cmds.add_cmd("inspect", "Return low-level information on a service")
-        self.cmds.add_cmd("install", "Install all the Ferry images")
+        self.cmds.add_cmd("install", "Install the Ferry images")
         self.cmds.add_cmd("logs", "Copy over the logs to the host")
         self.cmds.add_cmd("ps", "List deployed and running services")
         self.cmds.add_cmd("rm", "Remove a service or snapshot")
@@ -371,14 +371,25 @@ class CLI(object):
         if(cmd == 'start'):
             self._check_ssh_key()
 
+            if '-b' in options:
+                build_dir = options['-b'][0]
+                dockerfile = build_dir + '/Dockerfile'
+                names = self.installer._get_image(dockerfile)
+                name = names.pop().split("/")
+                if len(name) == 1:
+                    repo = GUEST_DOCKER_REPO
+                    image = name[0]
+                else:
+                    repo = name[0]
+                    image = name[1]
+
+                self.installer._compile_image(image, repo, build_dir, build=True)
+                if len(names) > 0:
+                    self.installer._tag_images(image, repo, names)
+
             arg = args.pop(0)
             json_arg = {}
-            if os.path.exists(arg):
-                json_string = self._read_file_arg(arg)
-                json_arg = json.loads(json_string)
-                json_arg['_file'] = arg
-                json_arg['_file_path'] = arg
-            else:
+            if not os.path.exists(arg):
                 # Check if the user wants to use one of the global plans.
                 global_path = FERRY_HOME + '/data/plans/' + arg
 
@@ -388,17 +399,21 @@ class CLI(object):
                 n, e = os.path.splitext(global_path)
                 if e == '':
                     file_path += '.yaml'
-                    e = 'yaml'
+            else:
+                file_path = arg
 
-                if os.path.exists(file_path):
-                    if e == 'json':
-                        json_string = self._read_file_arg(file_path)
-                        json_arg = json.loads(json_string)
-                    elif e == 'yaml':
-                        yaml_file = open(file_path, 'r')
-                        json_arg = yaml.load(yaml_file)
-                    json_arg['_file_path'] = file_path
-                json_arg['_file'] = arg
+            if os.path.exists(file_path):
+                file_path = os.path.abspath(file_path)
+                n, e = os.path.splitext(file_path)
+                if e == '.json':
+                    json_string = self._read_file_arg(file_path)
+                    json_arg = json.loads(json_string)
+                elif e == '.yaml':
+                    yaml_file = open(file_path, 'r')
+                    json_arg = yaml.load(yaml_file)
+                json_arg['_file_path'] = file_path
+
+            json_arg['_file'] = arg
             return self._create_stack(json_arg, args)
         elif(cmd == 'ps'):
             if len(args) > 0 and args[0] == '-a':
