@@ -44,6 +44,8 @@ class CassandraClientInitializer(object):
                             '/service/sbin/startnode %s %s' % (cmd, entry_point['storage_url']))
     def start_service(self, containers, entry_point, fabric):
         self._execute_service(containers, entry_point, fabric, "start")
+    def restart_service(self, containers, entry_point, fabric):
+        self._execute_service(containers, entry_point, fabric, "restart")
     def stop_service(self, containers, entry_point, fabric):
         self._execute_service(containers, entry_point, fabric, "stop")
 
