@@ -16,6 +16,7 @@
 import os
 import json
 import logging
+import re
 from subprocess import Popen, PIPE
 
 DOCKER_SOCK='unix:////var/run/ferry.sock'
@@ -29,9 +30,9 @@ class DockerInstance(object):
             self.host_name = None
             self.external_ip = None
             self.internal_ip = None
-            self.external_port = 0
-            self.internal_port = 0
+            self.ports = {}
             self.image = ''
+            self.keys = None
             self.volumes = None
             self.default_user = None
             self.name = None
@@ -42,13 +43,13 @@ class DockerInstance(object):
             self.host_name = json_data['hostname']
             self.external_ip = json_data['external_ip']
             self.internal_ip = json_data['internal_ip']
-            self.external_port = json_data['external_port']
-            self.internal_port = json_data['internal_port']
+            self.ports = json_data['ports']
             self.image = json_data['image']
-            self.volumes = json_data['volumes']
             self.default_user = json_data['user']
             self.name = json_data['name']
             self.args = json_data['args']
+            self.keys = json_data['keys']
+            self.volumes = json_data['volumes']
 
     """
     Return in JSON format. 
@@ -57,12 +58,12 @@ class DockerInstance(object):
         json_reply = { '_type' : 'docker',
                        'external_ip' : self.external_ip,
                        'internal_ip' : self.internal_ip,
-                       'external_port' : self.external_port,
-                       'internal_port' : self.internal_port,
+                       'ports' : self.ports,
                        'hostname' : self.host_name,
                        'container' : self.container,
                        'image' : self.image,
                        'type': self.service_type, 
+                       'keys' : self.keys,
                        'volumes' : self.volumes,
                        'user' : self.default_user,
                        'name' : self.name,
@@ -93,7 +94,8 @@ class DockerCLI(object):
         self.port_flag = ' -p'
         self.expose_flag = ' -expose'
         self.volume_flag = ' -v'
-        self.net_flag = ' -nb'
+        self.lxc_flag = ' -lxc-conf'
+        self.disable_net = ' -n=false'
         self.host_flag = ' -h'
         self.fs_flag = ' -s'
 
@@ -195,16 +197,15 @@ class DockerCLI(object):
     """
     Stop a running container
     """
-    def stop(self, container, phys_net=None, remove=False):
+    def stop(self, container):
         cmd = self.docker + ' ' + self.stop_cmd + ' ' + container
         logging.warning(cmd)
-        # Popen(cmd, stdout=PIPE, shell=True)
         output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
 
     """
     Remove a container
     """
-    def remove(self, container, phys_net=None, remove=False):
+    def remove(self, container):
         cmd = self.docker + ' ' + self.rm_cmd + ' ' + container
         logging.warning(cmd)
         output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
@@ -212,7 +213,7 @@ class DockerCLI(object):
     """
     Start a stopped container. 
     """
-    def start(self, container, service_type, args):
+    def start(self, container, service_type, keys, volumes, args):
         cmd = self.docker + ' ' + self.start_cmd + ' ' + container
         logging.warning(cmd)
         output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
@@ -220,6 +221,8 @@ class DockerCLI(object):
         # Now parse the output to get the IP and port
         container = output.strip()
         return self.inspect(container = container, 
+                            keys = keys,
+                            volumes = volumes,
                             service_type = service_type, 
                             args = args)
 
@@ -228,11 +231,8 @@ class DockerCLI(object):
     The Docker allocator will ignore subnet, volumes, instance_name, and key
     information since everything runs locally. 
     """
-    def run(self, service_type, image, volumes, keys, phys_net, security_group, expose_group=None, hostname=None, args=None):
+    def run(self, service_type, image, volumes, keys, security_group, expose_group=None, hostname=None, default_cmd=None, args=None, lxc_opts=None):
         flags = self.daemon 
-        if phys_net != None:
-            flags += self.net_flag
-            flags += ' %s ' % phys_net
 
         # Specify the hostname (this is optional)
         if hostname != None:
@@ -261,22 +261,46 @@ class DockerCLI(object):
         if keys != None:
             for v in keys.keys():
                 flags += self.volume_flag
-                flags += ' %s:%s' % (v, keys[v])
-            
-        # Now construct the final docker command. 
-        cmd = self.docker + ' ' + self.run_cmd + ' ' + flags + ' ' + image
-        logging.warning(cmd)
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+                # flags += ' %s:%s' % (v, keys[v])
+                flags += ' %s:%s' % (keys[v], v)
 
-        # Now parse the output to get the IP and port
-        container = output.strip()
-        return self.inspect(container, volumes, hostname, service_type, args)
+        # Add the lxc options
+        if lxc_opts != None:
+            flags += self.disable_net
+            for o in lxc_opts:
+                flags += self.lxc_flag
+                flags += ' \"%s\"' % o
+
+        if not default_cmd:
+            default_cmd = ''
+
+        # Now construct the final docker command. 
+        cmd = self.docker + ' ' + self.run_cmd + ' ' + flags + ' ' + image + ' ' + default_cmd
+        logging.warning(cmd)
+        child = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
+
+        err = child.stderr.read().strip()
+        if re.compile('[/:\s\w]*Can\'t connect[\'\s\w]*').match(err):
+            logging.error("Ferry docker daemon does not appear to be running")
+            return None
+        elif re.compile('Unable to find image[\'\s\w]*').match(err):
+            logging.error("%s not present" % image)
+            return None
+
+        container = child.stdout.read().strip()
+        return self.inspect(container, keys, volumes, hostname, service_type, args)
+
+    def _get_lxc_net(self, lxc_tuples):
+        for l in lxc_tuples:
+            if l['Key'] == 'lxc.network.ipv4':
+                return l['Value']
+        return None
 
     """
     Inspect a container and return information on how
     to connect to the container. 
     """
-    def inspect(self, container, volumes=None, hostname=None, service_type=None, args=None):
+    def inspect(self, container, keys=None, volumes=None, hostname=None, service_type=None, args=None):
         cmd = self.docker + ' ' + self.inspect_cmd + ' ' + container
         logging.warning(cmd)
 
@@ -292,6 +316,11 @@ class DockerCLI(object):
         instance.container = data['ID']
         instance.internal_ip = data['NetworkSettings']['IPAddress']
 
+        # If we've used the lxc config, then the networking information
+        # will be located somewhere else. 
+        if instance.internal_ip == "":
+            instance.internal_ip = self._get_lxc_net(data['HostConfig']['LxcConf'])
+
         if hostname:
             instance.host_name = hostname
         else:
@@ -301,16 +330,16 @@ class DockerCLI(object):
         instance.service_type = service_type
         instance.args = args
 
-        port_mapping = data['NetworkSettings']['PortMapping']
+        port_mapping = data['HostConfig']['PortBindings']
         if port_mapping:
-            for origin in port_mapping:
-                instance.internal_port = origin
-                instance.external_port = port_mapping[origin]
+            instance.ports = port_mapping
 
         # Add any data volume information. 
         if volumes:
             instance.volumes = volumes
         else:
-            # Need to inspect to get the volume bindings. 
             instance.volumes = data['Volumes']
+
+        if keys:
+            instance.keys = keys
         return instance
