@@ -14,10 +14,12 @@
 #
 
 import ferry.install
+import json
 import logging
 import time
 from subprocess import Popen, PIPE
 from ferry.docker.docker import DockerCLI
+from ferry.ip.client import DHCPClient
 
 """
 Allocate local docker instances
@@ -27,6 +29,22 @@ class DockerFabric(object):
         self.repo = 'public'
         self.docker_user = 'root'
         self.cli = DockerCLI()
+        self.network = DHCPClient(self._get_gateway())
+
+    def _get_gateway(self):
+        cmd = "ifconfig drydock0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'"
+        gw = Popen(cmd, stdout=PIPE, shell=True).stdout.read().strip()
+
+        cmd = "ifconfig drydock0 | grep 'inet addr:' | cut -d: -f4 | awk '{ print $1}'"
+        netmask = Popen(cmd, stdout=PIPE, shell=True).stdout.read().strip()
+        mask = map(int, netmask.split("."))
+        cidr = 1
+        if mask[3] == 0:
+            cidr = 8
+        if mask[2] == 0:
+            cidr *= 2
+
+        return "%s/%d" % (gw, 32 - cidr)
 
     """
     Read the location of the directory containing the keys
@@ -53,11 +71,16 @@ class DockerFabric(object):
     Restart the stopped containers.
     """
     def restart(self, container_info):
-        container = self.cli.start(container_info['container'],
-                                   container_info['type'],
-                                   container_info['args'])
-        container.default_user = self.docker_user
-        return container
+        containers = []
+        for c in container_info:
+            container = self.cli.start(c['container'],
+                                       c['type'],
+                                       c['keys'],
+                                       c['volumes'],
+                                       c['args'])
+            container.default_user = self.docker_user
+            containers.append(container)
+        return containers
 
     """
     Allocate several instances.
@@ -66,37 +89,54 @@ class DockerFabric(object):
         containers = []
         mounts = {}
         for c in container_info:
+            # Get a new IP address for this container and construct
+            # a default command. 
+            ip = self.network.assign_ip(c)
+            gw = self._get_gateway().split("/")[0]
+
+            lxc_opts = ["lxc.network.type = veth",
+                        "lxc.network.ipv4 = %s" % ip, 
+                        "lxc.network.ipv4.gateway = %s" % gw,
+                        "lxc.network.link = drydock0",
+                        "lxc.network.name = eth0",
+                        "lxc.network.flags = up"]
+
+            c['default_cmd'] = "/service/sbin/startnode init"
+
             # Start a container with a specific image, in daemon mode,
             # without TTY, and on a specific port
             container = self.cli.run(service_type = c['type'], 
                                      image = c['image'], 
                                      volumes = c['volumes'],
                                      keys = c['keys'], 
-                                     phys_net = None, 
                                      security_group = c['ports'],
                                      expose_group = c['exposed'], 
                                      hostname = c['hostname'],
-                                     args= c['args'])
-            container.default_user = self.docker_user
-            containers.append(container)
+                                     default_cmd = c['default_cmd'],
+                                     args= c['args'],
+                                     lxc_opts = lxc_opts)
+            if container:
+                container.default_user = self.docker_user
+                container.internal_ip = ip
+                containers.append(container)
+                self.network.set_owner(ip, container.container)
 
-            # Not all containers have a unique name. 
-            if 'name' in c:
-                container.name = c['name']
+                if 'name' in c:
+                    container.name = c['name']
 
-            if 'volume_user' in c:
-                mounts[container] = {'user':c['volume_user'],
-                                     'vols':c['volumes'].items()}
+                if 'volume_user' in c:
+                    mounts[container] = {'user':c['volume_user'],
+                                         'vols':c['volumes'].items()}
 
-        # We should wait for a second to let the ssh server start
-        # on the containers (otherwise sometimes we get a connection refused)
-        time.sleep(2)
+                # We should wait for a second to let the ssh server start
+                # on the containers (otherwise sometimes we get a connection refused)
+                time.sleep(2)
 
-        # Check if we need to set the file permissions
-        # for the mounted volumes. 
-        for c, i in mounts.items():
-            for _, v in i['vols']:
-                self.cmd([c], 'chown -R %s %s' % (i['user'], v))
+                # Check if we need to set the file permissions
+                # for the mounted volumes. 
+                for c, i in mounts.items():
+                    for _, v in i['vols']:
+                        self.cmd([c], 'chown -R %s %s' % (i['user'], v))
 
         return containers
 
@@ -112,6 +152,7 @@ class DockerFabric(object):
     """
     def remove(self, containers):
         for c in containers:
+            self.network.free_ip(c.internal_ip)
             self.cli.remove(c.container)
 
     """
@@ -128,7 +169,8 @@ class DockerFabric(object):
                                'base' : c.image,
                                'type' : c.service_type, 
                                'name' : c.name, 
-                               'args' : c.args} )
+                               'args' : c.args,
+                               'ports': c.ports} )
             self.cli.commit(c, snapshot_name)
         return snapshots
 
@@ -144,7 +186,8 @@ class DockerFabric(object):
                               'base' : c.image,
                               'type' : c.service_type, 
                               'name' : c.name, 
-                              'args' : c.args} )
+                              'args' : c.args,
+                              'ports': c.ports} )
             if not registry:
                 self.cli.commit(c, image_name)
             else:
