@@ -26,6 +26,7 @@ import sys
 import time
 import uuid
 from pymongo import MongoClient
+from sets import Set
 from ferry.install import *
 from ferry.docker.docker        import DockerInstance
 from ferry.docker.fabric        import DockerFabric
@@ -36,20 +37,34 @@ class DockerManager(object):
     SSH_PORT = 22
 
     def __init__(self):
-        # Image names
-        self.DOCKER_GLUSTER = DEFAULT_DOCKER_REPO + '/gluster'
-        self.DOCKER_HADOOP = DEFAULT_DOCKER_REPO + '/hadoop'
-        self.DOCKER_HADOOP_CLIENT = DEFAULT_DOCKER_REPO + '/hadoop-client'
-        self.DOCKER_HIVE = DEFAULT_DOCKER_REPO + '/hive-metastore'
-        self.DOCKER_CASSANDRA = DEFAULT_DOCKER_REPO + '/cassandra'
-        self.DOCKER_CASSANDRA_CLIENT = DEFAULT_DOCKER_REPO + '/cassandra-client'
-        self.DOCKER_TITAN = DEFAULT_DOCKER_REPO + '/titan'
-        self.DOCKER_MPI = DEFAULT_DOCKER_REPO + '/openmpi'
-        self.DOCKER_MPI_CLIENT = DEFAULT_DOCKER_REPO + '/openmpi'
-
         # Generate configuration.
         self.config = ConfigFactory()
 
+        # Service mappings
+        self.service = {
+            'openmpi' : { 
+                'server' : self.config.mpi,
+                'client' : self.config.mpi_client },
+            'yarn': { 
+                'server' : self.config.yarn,
+                'client' : self.config.hadoop_client },
+            'gluster': { 
+                'server' : self.config.gluster,
+                'client' : self.config.mpi_client},
+            'cassandra': { 
+                'server' : self.config.cassandra,
+                'client' : self.config.cassandra_client},
+            'titan': { 
+                'server' : self.config.titan,
+                'client' : self.config.cassandra_client},
+            'hadoop': { 
+                'server' : self.config.hadoop,
+                'client' : self.config.hadoop_client},
+            'hive': { 
+                'server' : self.config.hadoop,
+                'client' : self.config.hadoop_client}
+            }
+
         # Docker tools
         self.docker = DockerFabric()
         self.deploy = DeployEngine(self.docker)
@@ -131,37 +146,37 @@ class DockerManager(object):
 
     def _get_snapshot_info(self, stack_uuid):
         v = self.cluster_collection.find_one( {'uuid' : stack_uuid} )
-        s = self.snapshot_collection.find_one( {'snapshot_uuid':v['snapshot_uuid']} )
-        if s:
-            time = s['snapshot_ts'].strftime("%m/%w/%Y (%I:%M %p)")
-            return { 'snapshot_ts' : time,
-                     'snapshot_uuid' : v['snapshot_uuid'] }
+        if v:
+            s = self.snapshot_collection.find_one( {'snapshot_uuid':v['snapshot_uuid']} )
+            if s:
+                time = s['snapshot_ts'].strftime("%m/%w/%Y (%I:%M %p)")
+                return { 'snapshot_ts' : time,
+                         'snapshot_uuid' : v['snapshot_uuid'] }
 
     def _get_service(self, service_type):
-        service = None
-        if service_type == 'mpi':
-            service = self.config.mpi
-        elif service_type == 'yarn':
-            service = self.config.yarn
-        elif service_type == 'gluster':
-            service = self.config.gluster
-        elif service_type == 'cassandra':
-            service = self.config.cassandra
-        elif service_type == 'titan':
-            service = self.config.titan
-        elif service_type == 'hadoop':
-            service = self.config.hadoop
-        elif service_type == 'hive':
-            service = self.config.hadoop
-        elif service_type == 'hadoop-client':
-            service = self.config.hadoop_client
-        elif service_type == 'cassandra-client':
-            service = self.config.cass_client
-        elif service_type == 'mpi-client':
-            service = self.config.mpi_client
+        if service_type in self.service:
+            return self.service[service_type]['server']
         else:
             logging.error("unknown service " + service_type)
-        return service
+            return None
+
+    def _get_client_services(self, storage_entry, compute_entry):
+        """
+        Get a list of all the client types that are needed for the supplied
+        storage and compute backends. So, for example, if the user has specified
+        a Hadoop backend, then we'll need to supply the Hadoop client, etc. 
+        """
+        client_services = Set()
+        service_names = []
+        for storage in storage_entry:
+            if storage['type'] in self.service:
+                client_services.add(self.service[storage['type']]['client'])
+                service_names.append(storage['type'])
+        for compute in compute_entry:
+            if compute['type'] in self.service:
+                client_services.add(self.service[compute['type']]['client'])
+                service_names.append(compute['type'])
+        return client_services, service_names
 
     """
     Helper method to copy directories. shutil fails if the 
@@ -187,14 +202,10 @@ class DockerManager(object):
                 return
 
     def copy_logs(self, stack_uuid, to_dir):
-        connectors, compute, storage = self._get_cluster_instances(stack_uuid)
+        _, compute, storage = self._get_cluster_instances(stack_uuid)
         storage_dir = to_dir + '/' + stack_uuid + '/storage'
         compute_dir = to_dir + '/' + stack_uuid + '/compute'
-        connector_dir = to_dir + '/' + stack_uuid + '/connectors'
 
-        for c in connectors:
-            for i in c['instances']:
-                self._copy_instance_logs(i, connector_dir)
         for c in compute:
             for i in c['instances']:
                 self._copy_instance_logs(i, compute_dir)
@@ -231,7 +242,7 @@ class DockerManager(object):
 
         storage_uuids = []
         compute_uuids = []
-        if cluster and 'baackends' in cluster:
+        if cluster and 'backends' in cluster:
             for b in cluster['backends']['uuids']:
                 if b['storage'] != None:
                     storage_uuids.append(b['storage'])
@@ -334,15 +345,28 @@ class DockerManager(object):
     Allocate new UUIDs. 
     """
     def _new_service_uuid(self):
-        services = self.service_collection.find()
-        return "se-" + str(services.count())
+        while True:
+            longid = str(uuid.uuid4())
+            shortid = 'se-' + longid.split('-')[0]
+            services = self.service_collection.find_one( {'uuid' : shortid} )
+            if not services:
+                return shortid
 
     def _new_stack_uuid(self):
-        clusters = self.cluster_collection.find()
-        return "sa-" + str(clusters.count())
+        while True:
+            longid = str(uuid.uuid4())
+            shortid = 'sa-' + longid.split('-')[0]
+            services = self.cluster_collection.find_one( {'uuid' : shortid} )
+            if not services:
+                return shortid
 
     def _new_snapshot_uuid(self, cluster_uuid):
-        return "sn-%s-%s" % (cluster_uuid, str(uuid.uuid4()))
+        while True:
+            longid = str(uuid.uuid4())
+            shortid = 'sn-' + longid.split('-')[0]
+            services = self.snapshot_collection.find_one( {'snapshot_uuid' : shortid} )
+            if not services:
+                return shortid
 
     """
     Determine if the supplied UUID is a valid snapshot. 
@@ -465,7 +489,8 @@ class DockerManager(object):
     def _read_key_dir(self):
         f = open(ferry.install.DEFAULT_DOCKER_KEY, 'r')
         k = f.read().strip().split("://")
-        return { k[1] : '/service/keys' }
+        # return { k[1] : '/service/keys' }
+        return { '/service/keys' : k[1]  }
 
     """
     Prepare the environment for storage containers.
@@ -484,12 +509,8 @@ class DockerManager(object):
         plan = {'localhost':{'containers':[]}}
 
         # Get the actual number of containers needed. 
-        if storage_type == 'gluster':
-            instances = self.config.gluster.get_total_instances(num_instances, layers)
-        elif storage_type == 'cassandra':
-            instances = self.config.cassandra.get_total_instances(num_instances, layers)
-        elif storage_type == 'hadoop':
-            instances = self.config.hadoop.get_total_instances(num_instances, layers)
+        storage_service = self._get_service(storage_type)
+        instances = storage_service.get_total_instances(num_instances, layers)
 
         # Now get the new container-specific information. 
         i = 0
@@ -537,10 +558,8 @@ class DockerManager(object):
         plan = {'localhost':{'containers':[]}}
 
         # Get the actual number of containers needed. 
-        if compute_type == 'yarn':
-            instances = self.config.yarn.get_total_instances(num_instances, layers)
-        elif compute_type == 'mpi':
-            instances = self.config.mpi.get_total_instances(num_instances, layers)
+        compute_service = self._get_service(compute_type)
+        instances = compute_service.get_total_instances(num_instances, layers)
 
         i = 0
         for t in instances:
@@ -567,29 +586,10 @@ class DockerManager(object):
     service, then just use the raw image. Otherwise, look for a snapshot image. 
     """
     def _get_instance_image(self, instance_type, uuid=None):
-        image = None
-        if instance_type == 'hadoop-client':
-            image = self.DOCKER_HADOOP_CLIENT
-        elif instance_type == 'cassandra-client':
-            image = self.DOCKER_CASSANDRA_CLIENT
-        elif instance_type == 'mpi-client':
-            image = self.DOCKER_MPI_CLIENT
-        elif instance_type == 'gluster':
-            image = self.DOCKER_GLUSTER
-        elif instance_type == 'cassandra':
-            image = self.DOCKER_CASSANDRA
-        elif instance_type == 'titan':
-            image = self.DOCKER_TITAN
-        elif instance_type == 'hadoop':
-            image = self.DOCKER_HADOOP
-        elif instance_type == 'hive':
-            image = self.DOCKER_HIVE
-        elif instance_type == 'mpi':
-            image = self.DOCKER_MPI
-        elif instance_type == 'yarn':
-            image = self.DOCKER_HADOOP
-
-        return image
+        s = instance_type.split('/')
+        if len(s) == 1:
+            instance_type = DEFAULT_DOCKER_REPO + '/' + instance_type
+        return instance_type
 
     """
     Prepare the environment for connector containers.
@@ -599,29 +599,28 @@ class DockerManager(object):
                                        connector_type, 
                                        instance_type=None,
                                        name=None, 
+                                       ports=[],
                                        args=None):
-        ports = []
-        exposed = []
         plan = {'localhost':{'containers':[]}}
 
         # Determine the instance type from the connector type. 
         if not instance_type:
             instance_type = self._get_instance_image(connector_type)
 
-        service = self._get_service(connector_type)
-        container_dir, log_dir, host_name, ports, exposed = self._get_service_environment(service, 0, 1)
-        new_log_dir = self._new_log_dir(service_uuid, connector_type, 0)
-        dir_info = { new_log_dir : log_dir }
+        if not name:
+            host_name = "client-%s" % str(service_uuid)
+        else:
+            host_name = name
         container_info = { 'image':instance_type,
-                           'volumes':dir_info,
-                           'volume_user':DEFAULT_FERRY_OWNER, 
                            'keys': self._read_key_dir(), 
+                           'volumes':{},
                            'type':connector_type, 
                            'ports':ports,
-                           'exposed':exposed, 
+                           'exposed':[], 
                            'hostname':host_name,
                            'name':name, 
                            'args':args}
+
         plan['localhost']['containers'].append(container_info)
         return plan
 
@@ -799,6 +798,21 @@ class DockerManager(object):
             self.cluster_collection.update( {'uuid':cluster_uuid}, 
                                              {"$set": cluster_state } )
 
+    """
+    Restart an stopped compute cluster.
+    """
+    def restart_compute(self, service_uuid, container_info, compute_type):
+        containers = self._restart_containers(container_info)
+        container_info = self._serialize_containers(containers)
+
+        service = {'uuid':service_uuid, 
+                   'containers':container_info, 
+                   'status':'running'}
+        self._update_service_configuration(service_uuid, service)
+
+        self._restart_service(service_uuid, containers, compute_type)
+        return service_uuid        
+
     """
     Allocate a new compute cluster.
     """
@@ -831,11 +845,14 @@ class DockerManager(object):
 
         # Now copy over the configuration.
         self._transfer_config(config_dirs)
+        for k in self._read_key_dir().keys():
+            keydir = k
 
         container_info = self._serialize_containers(containers)
         service = {'uuid':service_uuid, 
                    'containers':container_info, 
                    'class':'compute',
+                   'keys': keydir, 
                    'type':compute_type,
                    'entry':entry_point,
                    'storage':storage_uuid, 
@@ -844,25 +861,61 @@ class DockerManager(object):
 
         # After the docker instance start, we need to start the
         # actual storage service (gluster, etc.). 
-        self._start_service(service_uuid, containers, compute_type)
+        self._start_service(service_uuid, containers, compute_type, entry_point)
         return service_uuid
         
     def _start_service(self,
                        uuid,
                        containers,
-                       service_type):
-        entry_point = self._get_service_configuration(uuid)
-        service = self._get_service(service_type)
+                       service_type,
+                       entry_point, 
+                       service=None):
+        if not service:
+            service = self._get_service(service_type)
         service.start_service(containers, entry_point, self.docker)
 
+    def _restart_service(self,
+                         uuid,
+                         containers,
+                         service_type):
+        service_info = self._get_service_configuration(uuid, detailed=True)
+        entry_point = service_info['entry']        
+        if service_info['class'] != 'connector':
+            service = self._get_service(service_type)
+            service.restart_service(containers, entry_point, self.docker)
+        else:
+            for backend in service_info['backends']:
+                service = self.service[backend]['client']
+                service.restart_service(containers, entry_point, self.docker)
+
     def _stop_service(self,
                       uuid,
                       containers,
                       service_type):
-        entry_point = self._get_service_configuration(uuid)
-        service = self._get_service(service_type)
-        service.stop_service(containers, entry_point, self.docker)
+        service_info = self._get_service_configuration(uuid, detailed=True)
+        entry_point = service_info['entry']        
+        if service_info['class'] != 'connector':
+            service = self._get_service(service_type)
+            service.stop_service(containers, entry_point, self.docker)
+        else:
+            for backend in service_info['backends']:
+                service = self.service[backend]['client']
+                service.stop_service(containers, entry_point, self.docker)
+    """
+    Restart an stopped storage cluster.
+    """
+    def restart_storage(self, service_uuid, container_info, storage_type):
+        containers = self._restart_containers(container_info)
+        container_info = self._serialize_containers(containers)
 
+        service = {'uuid':service_uuid, 
+                   'containers':container_info, 
+                   'status':'running'}
+        self._update_service_configuration(service_uuid, service)
+
+        self._restart_service(service_uuid, containers, storage_type)
+        return service_uuid        
+                        
     """
     Create a storage cluster and start a particular
     personality on that cluster. 
@@ -904,7 +957,7 @@ class DockerManager(object):
 
         # After the docker instance start, we need to start the
         # actual storage service (gluster, etc.). 
-        self._start_service(service_uuid, containers, storage_type)
+        self._start_service(service_uuid, containers, storage_type, entry_point)
         return service_uuid
 
     """
@@ -990,9 +1043,24 @@ class DockerManager(object):
     Lookup the stopped backend info. 
     """
     def fetch_stopped_backend(self, uuid):
-        service = self.cluster_collection.find_one( {'uuid':uuid} )
-        if service:
-            return service['backends']['backend']
+        cluster = self.cluster_collection.find_one( {'uuid':uuid} )
+        if cluster:
+            backends = []
+            for i, uuid in enumerate(cluster['backends']['uuids']):
+                storage_uuid = uuid['storage']
+                storage_conf = self._get_service_configuration(storage_uuid, 
+                                                               detailed=True)
+
+                compute_confs = []
+                if 'compute' in uuid:
+                    for c in uuid['compute']:
+                        compute_conf = self._get_service_configuration(c, detailed=True)
+                        compute_confs.append(compute_conf)
+                
+                backends.append( {'storage' : storage_conf,
+                                  'compute' : compute_confs} )
+            return backends
+            
 
     """
     Lookup the snapshot backend info. 
@@ -1028,13 +1096,7 @@ class DockerManager(object):
                 # contain the container ID. 
                 s = self._get_service_configuration(cuid, detailed=True)
                 if s and 'containers' in s:
-                    for c in s['containers']:
-                        connector_info.append(self.restart_connector(service_uuid = app_uuid,
-                                                                     connector_type = c['type'],
-                                                                     backend = backend_info,
-                                                                     name = c['name'], 
-                                                                     args = c['args'],
-                                                                     container = c['container']))
+                    connector_info.append(self._restart_connectors(cuid, s['containers'], backend_info))
         return connector_info
 
     """
@@ -1054,6 +1116,7 @@ class DockerManager(object):
                                                               backend = backend_info,
                                                               name = c['name'], 
                                                               args = c['args'],
+                                                              ports = s['ports'].keys(),
                                                               image = c['image']))
         return connector_info
                 
@@ -1071,6 +1134,7 @@ class DockerManager(object):
                                                               backend = backend_info,
                                                               name = s['name'], 
                                                               args = s['args'],
+                                                              ports = s['ports'].keys(),
                                                               image = s['image']))
         return connector_info
                 
@@ -1078,20 +1142,10 @@ class DockerManager(object):
     """
     Restart a stopped connector with an existing storage service. 
     """
-    def restart_connector(self,
-                          service_uuid, 
-                          connector_type, 
-                          backend=None,
-                          name=None, 
-                          args=None,
-                          container=None):
-        service = self._get_service(connector_type)
-
-        # Allocate all the containers. 
-        container = self._restart_containers({ 'container': container,
-                                               'type' : connector_type,
-                                               'args' : args})
-
+    def _restart_connectors(self,
+                            service_uuid, 
+                            container_info, 
+                            backend=None):
         # Initialize the connector and connect to the storage. 
         storage_entry = []
         compute_entry = []
@@ -1102,36 +1156,40 @@ class DockerManager(object):
                 for c in b['compute']:
                     compute_entry.append(self._get_service_configuration(c))
 
+        # Start the connectors.
+        connector_type = container_info[0]['type']
+        args = container_info[0]['args']
+        connectors = self._restart_containers(container_info)
+
         # Generate the environment variables that will be 
         # injected into the containers. 
         env_vars = self.config.generate_env_vars(storage_entry,
                                                  compute_entry)
+        services, backend_names = self._get_client_services(storage_entry, compute_entry)
+        entry_points = {}
+        for service in services:
+            config_dirs, entry_point = self.config.generate_connector_configuration(service_uuid, 
+                                                                                    connectors, 
+                                                                                    service,
+                                                                                    storage_entry,
+                                                                                    compute_entry,
+                                                                                    args)
+            # Merge all the entry points. 
+            entry_points = dict(entry_point.items() + entry_points.items())
 
-        # Now generate the configuration files that will be
-        # transferred to the containers. 
-        config_dirs, entry_point = self.config.generate_connector_configuration(service_uuid, 
-                                                                                [container], 
-                                                                                service,
-                                                                                storage_entry,
-                                                                                compute_entry,
-                                                                                args)
-        # Now copy over the configuration.
-        self._transfer_config(config_dirs)
-        self._transfer_env_vars([container], env_vars)
+            # Now copy over the configuration.
+            self._transfer_config(config_dirs)
+            self._transfer_env_vars(connectors, env_vars)
 
-        # Update the connector state. 
-        container_info = self._serialize_containers([container])
+        # Start the containers and update the state. 
+        self._restart_service(service_uuid, connectors, connector_type)
+        container_info = self._serialize_containers(connectors)
         service = {'uuid':service_uuid, 
                    'containers':container_info, 
-                   'class':'connector',
-                   'type':connector_type,
-                   'entry':entry_point,
-                   'uniq': name, 
+                   'backends':backend_names, 
+                   'entry':entry_points,
                    'status':'running'}
         self._update_service_configuration(service_uuid, service)
-
-        # Start the connector personality. 
-        self._start_service(service_uuid, [container], connector_type)
         return service_uuid
                 
     """
@@ -1142,23 +1200,8 @@ class DockerManager(object):
                            backend=None,
                            name=None, 
                            args=None,
+                           ports=None, 
                            image=None):
-        # Allocate a UUID.
-        service_uuid = self._new_service_uuid()
-        service = self._get_service(connector_type)
-
-        # Generate the data volumes. This basically defines which
-        # directories on the host get mounted in the container. 
-        plan = self._prepare_connector_environment(service_uuid = service_uuid, 
-                                                   connector_type = connector_type, 
-                                                   instance_type = image,
-                                                   name = name,
-                                                   args = args)
-
-
-        # Allocate all the containers. 
-        containers = self._start_containers(plan)
-
         # Initialize the connector and connect to the storage. 
         storage_entry = []
         compute_entry = []
@@ -1175,31 +1218,48 @@ class DockerManager(object):
         env_vars = self.config.generate_env_vars(storage_entry,
                                                  compute_entry)
 
+        # Allocate a UUID.
+        service_uuid = self._new_service_uuid()
+        plan = self._prepare_connector_environment(service_uuid = service_uuid, 
+                                                   connector_type = connector_type, 
+                                                   instance_type = image,
+                                                   name = name,
+                                                   ports = ports, 
+                                                   args = args)
+        containers = self._start_containers(plan)
+
         # Now generate the configuration files that will be
         # transferred to the containers. 
-        config_dirs, entry_point = self.config.generate_connector_configuration(service_uuid, 
-                                                                                containers, 
-                                                                                service,
-                                                                                storage_entry,
-                                                                                compute_entry,
-                                                                                args)
-        # Now copy over the configuration.
-        self._transfer_config(config_dirs)
-        self._transfer_env_vars(containers, env_vars)
+        entry_points = {}
+        services, backend_names = self._get_client_services(storage_entry, compute_entry)
+        for service in services:
+            config_dirs, entry_point = self.config.generate_connector_configuration(service_uuid, 
+                                                                                    containers, 
+                                                                                    service,
+                                                                                    storage_entry,
+                                                                                    compute_entry,
+                                                                                    args)
+            # Merge all the entry points. 
+            entry_points = dict(entry_point.items() + entry_points.items())
+
+            # Now copy over the configuration.
+            self._transfer_config(config_dirs)
+            self._transfer_env_vars(containers, env_vars)
+
+            # Start the connector personality. 
+            self._start_service(service_uuid, containers, connector_type, entry_point, service)
 
         # Update the connector state. 
         container_info = self._serialize_containers(containers)
-        service = {'uuid':service_uuid, 
-                   'containers':container_info, 
-                   'class':'connector',
-                   'type':connector_type,
-                   'entry':entry_point,
-                   'uniq': name, 
-                   'status':'running'}
-        self._update_service_configuration(service_uuid, service)
-
-        # Start the connector personality. 
-        self._start_service(service_uuid, containers, connector_type)
+        service_info = {'uuid':service_uuid, 
+                        'containers':container_info, 
+                        'backends':backend_names, 
+                        'class':'connector',
+                        'type':connector_type,
+                        'entry':entry_points,
+                        'uniq': name, 
+                        'status':'running'}
+        self._update_service_configuration(service_uuid, service_info)
         return service_uuid
 
     """
