@@ -29,7 +29,7 @@ import time
 from distutils import spawn
 from string import Template
 from subprocess import Popen, PIPE
-from ferry.options import CmdHelp
+from ferry.ip.client import DHCPClient
 
 def _get_ferry_home():
     if 'FERRY_HOME' in os.environ:
@@ -57,20 +57,24 @@ def _supported_python():
     return sys.version_info[0] == 2
 
 def _touch_file(file_name, content, root=False):
-    f = open(file_name, 'w+')
-    f.write(content)
-    f.close()
+    try:
+        f = open(file_name, 'w+')
+        f.write(content)
+        f.close()
 
-    if root:
-        uid, gid = _get_ferry_user()
-        os.chown(file_name, uid, gid)
-        os.chmod(file_name, 0664)
+        if root:
+            uid, gid = _get_ferry_user()
+            os.chown(file_name, uid, gid)
+            os.chmod(file_name, 0664)
+    except IOError as e:
+        logging.error("Could not create %s.\n" % file_name)
 
 FERRY_HOME=_get_ferry_home()
 DEFAULT_IMAGE_DIR=FERRY_HOME + '/data/dockerfiles'
 DEFAULT_KEY_DIR=FERRY_HOME + '/data/key'
 GLOBAL_KEY_DIR=DEFAULT_KEY_DIR
 DEFAULT_DOCKER_REPO='ferry'
+GUEST_DOCKER_REPO='ferry-user'
 DEFAULT_FERRY_OWNER='ferry:docker'
 DOCKER_CMD='docker-ferry'
 DOCKER_SOCK='unix:////var/run/ferry.sock'
@@ -83,6 +87,8 @@ DEFAULT_DOCKER_LOG='/var/lib/ferry/docker.log'
 DEFAULT_DOCKER_KEY='/var/lib/ferry/keydir'
 
 class Installer(object):
+    def __init__(self):
+        self.network = DHCPClient()
 
     def _read_key_dir(self):
         f = open(ferry.install.DEFAULT_DOCKER_KEY, 'r')
@@ -148,9 +154,14 @@ class Installer(object):
             build = True
 
         if '-u' in options:
-            # We want to re-build all the images. 
-            logging.warning("performing forced rebuild")
-            self.build_from_dir(DEFAULT_IMAGE_DIR, DEFAULT_DOCKER_REPO, build)
+            if len(options['-u']) > 0 and options['-u'][0] != True:
+                logging.warning("performing select rebuild (%s)" % str(options['-u']))
+                self.build_from_list(options['-u'], 
+                                     DEFAULT_IMAGE_DIR,
+                                     DEFAULT_DOCKER_REPO, build, recurse=False)
+            else:
+                logging.warning("performing forced rebuild")
+                self.build_from_dir(DEFAULT_IMAGE_DIR, DEFAULT_DOCKER_REPO, build)
         else:
             # We want to be selective about which images
             # to rebuild. Useful if one image breaks, etc. 
@@ -217,14 +228,26 @@ class Installer(object):
         # Check if there are any other Mongo instances runnig.
         self._clean_web()
 
+        # self.docker = DockerFabric()
+        # container_info = {'image': DEFAULT_DOCKER_REPO + '/mongodb',
+        #                   'type':'mongodb', 
+        #                   'volumes': { DEFAULT_MONGO_DB : '/service/data',
+        #                                DEFAULT_MONGO_LOG : '/service/logs' },
+        #                   'volume_user':DEFAULT_FERRY_OWNER, 
+        #                   'keys': { '/service/keys' : keydir }, 
+        #                   'ports':[],
+        #                   'exposed':[], 
+        #                   'hostname':'ferry-state',
+        #                   'args':None}
+        # containers = self.docker.alloc(container_info)
+        # if len(containers) == 0:
+        #     exit(1)
+
         # Start the Mongo server.
-        mongo_data = '/service/data'
-        mongo_log = '/service/logs'
-        mongo_keys = '/service/keys'
         keydir, _ = self._read_key_dir()
-        cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' run -d -v %s:%s -v %s:%s -v %s:%s %s/mongodb' % (keydir, mongo_keys,
-                                                                                                     DEFAULT_MONGO_DB, mongo_data,
-                                                                                                     DEFAULT_MONGO_LOG, mongo_log,
+        cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' run -d -v %s:%s -v %s:%s -v %s:%s %s/mongodb' % (keydir, '/service/keys',
+                                                                                                     DEFAULT_MONGO_DB, '/service/data',
+                                                                                                     DEFAULT_MONGO_LOG, '/service/logs',
                                                                                                      DEFAULT_DOCKER_REPO)
         logging.warning(cmd)
         child = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
@@ -245,12 +268,23 @@ class Installer(object):
         ip = output_json[0]['NetworkSettings']['IPAddress']
         _touch_file('/tmp/mongodb.ip', ip, root=True)
 
+        # Set the MongoDB env. variable. 
+        my_env = os.environ.copy()
+        my_env['MONGODB'] = ip
+
         # Sleep a little while to let Mongo start receiving.
         time.sleep(2)
 
-        # Start the HTTP servers
-        my_env = os.environ.copy()
-        my_env['MONGODB'] = ip
+        # Start the DHCP server
+        logging.warning("starting dhcp server")
+        cmd = 'gunicorn -t 3600 -b 127.0.0.1:5000 -w 1 ferry.ip.dhcp:app &'
+        Popen(cmd, stdout=PIPE, shell=True, env=my_env)
+        time.sleep(2)
+
+        # Reserve the Mongo IP.
+        self.network.reserve_ip(ip)
+
+        # Start the Ferry HTTP servers
         logging.warning("starting http servers on port 4000 and mongo %s" % ip)
         cmd = 'gunicorn -e FERRY_HOME=%s -t 3600 -w 3 -b 127.0.0.1:4000 ferry.http.httpapi:app &' % FERRY_HOME
         Popen(cmd, stdout=PIPE, shell=True, env=my_env)
@@ -325,9 +359,9 @@ class Installer(object):
             build_images = []
             for f in os.listdir(image_dir):
                 dockerfile = image_dir + '/' + f + '/Dockerfile'
-                image_name = self._check_dockerfile(dockerfile, repo)
-                if image_name:
-                    build_images.append(image_name)
+                image_names = self._check_dockerfile(dockerfile, repo)
+                if len(image_names) > 0:
+                    build_images += image_names
             return build_images
         else:
             logging.error("ferry daemon not started")
@@ -335,7 +369,7 @@ class Installer(object):
     """
     Build the docker images
     """
-    def build_from_list(self, to_build, image_dir, repo, build=False):
+    def build_from_list(self, to_build, image_dir, repo, build=False, recurse=True):
         if self._docker_running():
             built_images = {}
             for f in os.listdir(image_dir):
@@ -343,10 +377,16 @@ class Installer(object):
 
             for f in os.listdir("/tmp/dockerfiles/"):
                 dockerfile = '/tmp/dockerfiles/' + f + '/Dockerfile'
-                image = self._get_image(dockerfile)
-                if image in to_build:
+                images = self._get_image(dockerfile)
+                intersection = [i for i in images if i in to_build]
+                if len(intersection) > 0:
+                    image = images.pop(0)
                     logging.warning("building image " + image)
-                    self._build_image(dockerfile, repo, built_images, recurse=True, build=build)
+                    self._build_image(image, dockerfile, repo, built_images, recurse=recurse, build=build)
+
+                    if len(images) > 0:
+                        logging.warning("tagging images " + image)
+                        self._tag_images(image, repo, images)
 
             # After building everything, get rid of the temp dir.
             shutil.rmtree("/tmp/dockerfiles")
@@ -363,7 +403,13 @@ class Installer(object):
                 self._transform_dockerfile(image_dir, f, repo)
             for f in os.listdir("/tmp/dockerfiles/"):
                 dockerfile = "/tmp/dockerfiles/" + f + "/Dockerfile"
-                self._build_image(dockerfile, repo, built_images, recurse=True, build=build)
+                images = self._get_image(dockerfile)
+                image = images.pop(0)
+                self._build_image(image, dockerfile, repo, built_images, recurse=True, build=build)
+
+                if len(images) > 0:
+                    logging.warning("tagging images " + image)
+                    self._tag_images(image, repo, images)
 
             # After building everything, get rid of the temp dir.
             shutil.rmtree("/tmp/dockerfiles")
@@ -374,15 +420,15 @@ class Installer(object):
         return os.path.exists('/var/run/ferry.sock')
 
     def _check_dockerfile(self, dockerfile, repo):
-        image = self._get_image(dockerfile)
-        qualified = DEFAULT_DOCKER_REPO + '/' + image
-
-        cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' inspect ' + qualified + ' 2> /dev/null'
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
-        if output.strip() == '[]':
-            return image
-        else:
-            return None
+        not_installed = []
+        images = self._get_image(dockerfile)
+        for image in images:
+            qualified = DEFAULT_DOCKER_REPO + '/' + image
+            cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' inspect ' + qualified + ' 2> /dev/null'
+            output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+            if output.strip() == '[]':
+                not_installed.append(image)
+        return not_installed
 
     def _transform_dockerfile(self, image_dir, f, repo):
         if not os.path.exists("/tmp/dockerfiles/" + f):
@@ -398,14 +444,14 @@ class Installer(object):
             out.write(s)
         out.close()
 
-    def _build_image(self, f, repo, built_images, recurse=False, build=False):
+    def _build_image(self, image, f, repo, built_images, recurse=False, build=False):
         base = self._get_base(f)
         if recurse and base != "base":
             image_dir = os.path.dirname(os.path.dirname(f))
             dockerfile = image_dir + '/' + base + '/Dockerfile'
-            self._build_image(dockerfile, repo, built_images, recurse, build)
+            self._build_image(base, dockerfile, repo, built_images, recurse, build)
 
-        image = os.path.dirname(f).split("/")[-1]
+        # image = os.path.dirname(f).split("/")[-1]
         if not image in built_images:
             if base == "base":
                 self._pull_image(base)
@@ -414,13 +460,14 @@ class Installer(object):
             self._compile_image(image, repo, os.path.dirname(f), build)
 
     def _get_image(self, dockerfile):
+        names = []
         for l in open(dockerfile, 'r'):
             if l.strip() != '':
                 s = l.split()
                 if len(s) > 0:
                     if s[0].upper() == 'NAME':
-                        return s[1].strip()
-        return None
+                        names.append(s[1].strip())
+        return names
 
     def _get_base(self, dockerfile):
         base = None
@@ -465,6 +512,15 @@ class Installer(object):
             image_name = "%s/%s" % (repo, image)
             self._pull_image(image_name)
 
+    def _tag_images(self, image, repo, alternatives):
+        for a in alternatives:
+            cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' tag' + ' %s/%s:%s %s/%s:%s' % (repo, image, ferry.__version__, repo, a, ferry.__version__)
+            logging.warning(cmd)
+            child = Popen(cmd, stdout=PIPE, shell=True)
+            cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' tag' + ' %s/%s:latest %s/%s:latest' % (repo, image, repo, a)
+            logging.warning(cmd)
+            child = Popen(cmd, stdout=PIPE, shell=True)
+
     def _clean_images(self):
         cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' | grep none | awk \'{print $1}\' | xargs ' + DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' rmi'
         Popen(cmd, stdout=PIPE, shell=True)
@@ -509,3 +565,18 @@ class Installer(object):
                 os.remove('/var/run/ferry.sock')
             except OSError:
                 pass
+
+    def _get_gateway(self):
+        cmd = "ifconfig drydock0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'"
+        gw = Popen(cmd, stdout=PIPE, shell=True).stdout.read().strip()
+
+        cmd = "ifconfig drydock0 | grep 'inet addr:' | cut -d: -f4 | awk '{ print $1}'"
+        netmask = Popen(cmd, stdout=PIPE, shell=True).stdout.read().strip()
+        mask = map(int, netmask.split("."))
+        cidr = 1
+        if mask[3] == 0:
+            cidr = 8
+        if mask[2] == 0:
+            cidr *= 2
+
+        return "%s/%d" % (gw, 32 - cidr)
